package jp.gr.java_conf.skrb.game.pong;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Random;

public class BallController {
    private PongFactory factory;

    private Sprite ball;

    private int initX;
    private int initY;
    private int oldX;
    private int oldY;

    private int diameter;
    private double initSpeed;
    private double maxSpeed;
    private double speedStep;
    private double direction;
    private double speed;

    private int stageWidth;
    private int stageHeight;

    private Random random;

    public BallController() {
        factory = PongFactory.getFactory();

	initX = Integer.getInteger("InitBallX").intValue();
	initY = Integer.getInteger("InitBallY").intValue();
	diameter = Integer.getInteger("BallDiameter").intValue();
//	initSpeed = Double.parseDouble(System.getProperty("InitBallSpeed"));
//	maxSpeed = Double.parseDouble(System.getProperty("MaxBallSpeed"));
//	speedStep = Double.parseDouble(System.getProperty("BallSpeedStep"));
	initSpeed = Double.valueOf(System.getProperty("InitBallSpeed")).doubleValue();
	maxSpeed = Double.valueOf(System.getProperty("MaxBallSpeed")).doubleValue();
	speedStep = Double.valueOf(System.getProperty("BallSpeedStep")).doubleValue();
	
        stageWidth = Integer.getInteger("StageWidth").intValue();
        stageHeight = Integer.getInteger("StageHeight").intValue();

	
	ball = factory.createBall(diameter, diameter);
        ball.moveTo(initX, initY);

        random = new Random();
        do {
            direction = random.nextDouble() * Math.PI;
        } while (direction < Math.PI / 6.0 
		 || direction > Math.PI / 6.0 * 5.0 
		 || direction == Math.PI / 2.0);

        speed = initSpeed;
    }

    public void init() {
        speed = initSpeed;
	ball = factory.createBall(diameter, diameter);
        ball.moveTo(initX, initY);
    }

    public Sprite getBall() {
	return ball;
    }

    private void bounce(int y) {
        direction = - direction + random.nextDouble() * 0.2 - 0.1;
	ball.moveTo(ball.getX(), y - ball.getHeight());
    }

    public boolean bounceCheck(Rectangle rect) {
	Rectangle old = new Rectangle(oldX, oldY, ball.getWidth(), ball.getHeight());
	Rectangle now = new Rectangle(ball.getX(), ball.getY(), ball.getWidth(), ball.getHeight());

	if (!contains(old, rect) && contains(now, rect)) {
	    bounce(rect.y);
	    return true;
	}

	return false;
    }
    
    private boolean contains(Rectangle target, Rectangle rect) {
        if (rect.y + rect.height >= target.y && rect.y <= target.y + target.height) {
            if (rect.x + rect.width >= target.x && rect.x <= target.x + target.width) {
                return true;
            }
        }

        return false;
    }

    public boolean update() {
	int x = ball.getX();
	int width = ball.getWidth();
	oldX = x;

        x += (int)(speed * Math.cos(direction));
        if (x > stageWidth - width) {
            direction = Math.PI - direction;
        } else if (x < 0) {
            direction = Math.PI - direction;
        }

	int y = ball.getY();
	int height = ball.getHeight();
	oldY = y;
        y += (int)(speed * Math.sin(direction));
        if (y > stageHeight - height) {
            return false;
        } else if (y < 0) {
            direction = - direction;

            if (speed < maxSpeed) {
                speed += speedStep;
            }
        }

        ball.moveTo(x, y);
        return true;
    }
}
