package jp.gr.java_conf.skrb.game.pong.awt;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import jp.gr.java_conf.skrb.game.pong.InputEvent;
import jp.gr.java_conf.skrb.game.pong.InputListener;
import jp.gr.java_conf.skrb.game.pong.PongView;
import jp.gr.java_conf.skrb.game.pong.Sprite;

public class AWTPongView implements PongView {
    protected Frame frame;
    protected Canvas canvas;
    protected BufferStrategy strategy;
    
    private List listeners;

    private int stageWidth;
    private int stageHeight;
    private int infoY;
    private int scoreX;
    private int highScoreX;
    private int restBallX;
    private int startMessageX;
    private int startMessageY;
    private String startMessage;

    public AWTPongView() {
	stageWidth = Integer.getInteger("StageWidth").intValue();
	stageHeight = Integer.getInteger("StageHeight").intValue();
	infoY = Integer.getInteger("InfoY").intValue();
	scoreX = Integer.getInteger("ScoreX").intValue();
	highScoreX = Integer.getInteger("HighScoreX").intValue();
	restBallX = Integer.getInteger("RestBallX").intValue();
	startMessageX = Integer.getInteger("StartMessageX").intValue();
	startMessageY = Integer.getInteger("StartMessageY").intValue();
	startMessage = System.getProperty("StartMessage");

	initFrame();

	listeners = Collections.synchronizedList(new ArrayList());
    }

    public void init() {
	initCanvas();

	frame.setVisible(true);

	Insets insets = frame.getInsets();
	frame.setSize(stageWidth + insets.left + insets.right,
		      stageHeight + insets.top + insets.bottom);
	canvas.setLocation(insets.left, insets.top);

	canvas.createBufferStrategy(2);
	strategy = canvas.getBufferStrategy();
    }

    protected void initFrame() {
        frame = new Frame("Pong");
        frame.setResizable(false);
	frame.setBounds(0, 0, stageWidth, stageHeight);
	frame.setIgnoreRepaint(false);
	frame.setLayout(null);

	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent event) {
		    fireGameQuitEvent(InputEvent.QUIT);
		}
	    });

	frame.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent event) {
                int code = event.getKeyCode();
                switch (code) {
                case KeyEvent.VK_Y:
                case KeyEvent.VK_SPACE:
		    fireGameStartEvent(InputEvent.START);
		    break;
                case KeyEvent.VK_Q:
		    fireGameQuitEvent(InputEvent.QUIT);
		    break;
                case KeyEvent.VK_RIGHT:
		    firePadMovingEvent(InputEvent.RIGHT);
                    break;
                case KeyEvent.VK_LEFT:
		    firePadMovingEvent(InputEvent.LEFT);
                    break;
                } 
            }
        });
    }

    protected void initCanvas() {
	canvas = new Canvas();
	canvas.setSize(stageWidth, stageHeight);
	frame.add(canvas);
    }

    public void addInputListener(InputListener listener) {
	listeners.add(listener);
    }

    public void removeInputListener(InputListener listener) {
	listeners.remove(listener);
    }

    private void fireGameStartEvent(int id) {
	InputEvent event = new InputEvent(id);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.get(i);
	    listener.gameStart(event);
	}
    }

    private void fireGameQuitEvent(int id) {
	InputEvent event = new InputEvent(id);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.get(i);
	    listener.gameQuit(event);
	}
    }

    private void firePadMovingEvent(int id) {
	InputEvent event = new InputEvent(id);

	for (int i = 0; i < listeners.size(); i++) {
	    InputListener listener = (InputListener)listeners.get(i);
	    listener.padMoving(event);
	}
    }

    protected void clear(Graphics g) {
	g.setColor(Color.BLACK);
	g.fillRect(0, 0, stageWidth, stageHeight);
    }

    protected void drawInfo(Graphics g, int rest, int score, int highScore) {
	g.setColor(Color.WHITE);
	g.drawString("Rest Ball: " + rest, restBallX, infoY);
	drawInfo(g, score, highScore);
    }

    protected void drawInfo(Graphics g, int score, int highScore) {
	g.setColor(Color.WHITE);
	g.drawString("Score: " + score, scoreX, infoY);
	g.drawString("High Score: " + highScore, highScoreX, infoY);
    }

    protected void drawStartMessage(Graphics g) {
	g.setColor(Color.WHITE);
	g.drawString(startMessage, startMessageX, startMessageY);
    }

    protected void drawSprites(Graphics g, Sprite ball, Sprite pad) {
	ball.draw(g);
	pad.draw(g);
    }

    public void draw(int rest, int score, int highScore, Sprite ball, Sprite pad, boolean gameFlag) {
	Graphics g = strategy.getDrawGraphics();
	
	if (!strategy.contentsLost()) {
	    clear(g);
	    drawInfo(g, rest, score, highScore);

	    if (!gameFlag) {
		drawStartMessage(g);
	    }
	    
	    drawSprites(g, ball, pad);

	    strategy.show();
	    g.dispose();
	}
    }

    public void end() {}
}
